@@ -6,13 +6,17 @@ import shutil
 import subprocess
 import sys
 
-version_info = (0, 1, 2)
+version_info = (0, 2, 3)
 __version__ = '.'.join(map(str, version_info))
 
 
 logger = logging.getLogger()
 
 
+if sys.version_info < (2, 6):
+    next = lambda gen: gen.next()
+
+
 class UserError(Exception):
     pass
 
@@ -43,14 +47,14 @@ def _virtualenv_sys(venv_path):
     executable = os.path.join(venv_path, 'bin', 'python')
     p = subprocess.Popen(['python',
         '-c', 'import sys;'
-              'print sys.version[:3];'
-              'print "\\n".join(sys.path);'],
+              'print (sys.version[:3]);'
+              'print ("\\n".join(sys.path));'],
         executable=executable,
         env={},
         stdout=subprocess.PIPE)
     stdout, err = p.communicate()
     assert not p.returncode and stdout
-    lines = stdout.splitlines()
+    lines = stdout.decode('utf-8').splitlines()
     return lines[0], filter(bool, lines[1:])
 
 
@@ -60,7 +64,8 @@ def clone_virtualenv(src_dir, dst_dir):
     if os.path.exists(dst_dir):
         raise UserError('dest dir %r exists' % dst_dir)
     #sys_path = _virtualenv_syspath(src_dir)
-    shutil.copytree(src_dir, dst_dir, symlinks=True)
+    shutil.copytree(src_dir, dst_dir, symlinks=True,
+            ignore=shutil.ignore_patterns('*.pyc'))
     version, sys_path = _virtualenv_sys(dst_dir)
     fixup_scripts(src_dir, dst_dir, version)
 
@@ -76,11 +81,14 @@ def clone_virtualenv(src_dir, dst_dir):
 
 def fixup_scripts(old_dir, new_dir, version, rewrite_env_python=False):
     bin_dir = os.path.join(new_dir, 'bin')
-    root, dirs, files = os.walk(bin_dir).next()
+    root, dirs, files = next(os.walk(bin_dir))
     for file_ in files:
         filename = os.path.join(root, file_)
-        if file_ == 'activate':
+        if 'activate' in file_:
             fixup_activate(os.path.join(root, file_), old_dir, new_dir)
+        elif file_.endswith('.pyc'):
+            # ignore compiled files
+            continue
         elif file_ in ['python', 'python%s' % version, 'activate_this.py']:
             continue
         elif os.path.islink(filename):
@@ -97,9 +105,8 @@ def fixup_script_(root, file_, old_dir, new_dir, version,
     env_shebang = '#!/usr/bin/env python'
 
     filename = os.path.join(root, file_)
-    f = open(filename, 'rb')
-    lines = f.readlines()
-    f.close()
+    with open(filename, 'rb') as f:
+        lines = f.readlines()
 
     if not lines:
         # warn: empty script
@@ -110,11 +117,12 @@ def fixup_script_(root, file_, old_dir, new_dir, version,
         shebang = new_shebang
         if version:
             shebang = shebang + version
+        shebang = (shebang + '\n').encode('utf-8')
         with open(filename, 'wb') as f:
-            f.write('%s\n' % shebang)
+            f.write(shebang)
             f.writelines(lines[1:])
 
-    bang = lines[0].strip()
+    bang = lines[0].decode('utf-8').strip()
 
     if not bang.startswith('#!'):
         return
@@ -135,13 +143,12 @@ def fixup_script_(root, file_, old_dir, new_dir, version,
 
 def fixup_activate(filename, old_dir, new_dir):
     logger.debug('fixing %s' % filename)
-    f = open(filename, 'rb')
-    data = f.read()
-    f.close()
+    with open(filename, 'rb') as f:
+        data = f.read().decode('utf-8')
+
     data = data.replace(old_dir, new_dir)
-    f = open(filename, 'wb')
-    f.write(data)
-    f.close()
+    with open(filename, 'wb') as f:
+        f.write(data.encode('utf-8'))
 
 
 def fixup_link(filename, old_dir, new_dir, target=None):
@@ -187,7 +194,7 @@ def fixup_syspath_items(syspath, old_dir, new_dir):
                 continue
         elif not _dirmatch(path, new_dir):
             continue
-        root, dirs, files = os.walk(path).next()
+        root, dirs, files = next(os.walk(path))
         for file_ in files:
             filename = os.path.join(root, file_)
             if filename.endswith('.pth'):
@@ -202,7 +209,7 @@ def fixup_pth_file(filename, old_dir, new_dir):
         lines = f.readlines()
     has_change = False
     for num, line in enumerate(lines):
-        line = line.strip()
+        line = line.decode('utf-8').strip()
         if not line or line.startswith('#') or line.startswith('import '):
             continue
         elif _dirmatch(line, old_dir):
@@ -216,11 +223,12 @@ def fixup_pth_file(filename, old_dir, new_dir):
 def fixup_egglink_file(filename, old_dir, new_dir):
     logger.debug('fixing %s' % filename)
     with open(filename, 'rb') as f:
-        link = f.read().strip()
+        link = f.read().decode('utf-8').strip()
     if _dirmatch(link, old_dir):
         link = link.replace(old_dir, new_dir, 1)
         with open(filename, 'wb') as f:
-            f.write('%s\n' % link)
+            link = (link + '\n').encode('utf-8')
+            f.write(link)
 
 
 def main():
@@ -229,14 +237,15 @@ def main():
     options, args = parser.parse_args()
     try:
         old_dir, new_dir = sys.argv[1:]
-    except ValueError, e:
+    except ValueError:
         parser.error("not enough arguments given.")
     old_dir = os.path.normpath(os.path.abspath(old_dir))
     new_dir = os.path.normpath(os.path.abspath(new_dir))
     logging.basicConfig(level=logging.WARNING)
     try:
         clone_virtualenv(old_dir, new_dir)
-    except UserError, e:
+    except UserError:
+        e = sys.exc_info()[1]
         parser.error(str(e))
 
 
